@@ -1,13 +1,18 @@
 import logging
-from typing import Callable, Optional, Tuple
+import sys
+from typing import Callable, Optional, Tuple, Any
 
 import numpy as np
 from numpy import ndarray
-from scipy.sparse import coo_matrix
 from skfem.element import DiscreteField, Element
 from skfem.mapping import Mapping
 from skfem.mesh import Mesh
 
+if "pyodide" in sys.modules:
+    from scipy.sparse.coo import coo_matrix
+else:
+    from scipy.sparse import coo_matrix
+
 from .abstract_basis import AbstractBasis
 from ..dofs import Dofs
 
@@ -39,11 +44,12 @@ class CellBasis(AbstractBasis):
                  elem: Element,
                  mapping: Optional[Mapping] = None,
                  intorder: Optional[int] = None,
-                 elements: Optional[ndarray] = None,
+                 elements: Optional[Any] = None,
                  quadrature: Optional[Tuple[ndarray, ndarray]] = None,
                  dofs: Optional[Dofs] = None):
-        """Combine :class:`~skfem.mesh.Mesh` and :class:`~skfem.element.Element`
-        into a set of precomputed global basis functions.
+        """Combine :class:`~skfem.mesh.Mesh` and
+        :class:`~skfem.element.Element` into a set of precomputed global basis
+        functions.
 
         Parameters
         ----------
@@ -69,25 +75,27 @@ class CellBasis(AbstractBasis):
         logger.info("Initializing {}({}, {})".format(type(self).__name__,
                                                      type(mesh).__name__,
                                                      type(elem).__name__))
-        super(CellBasis, self).__init__(mesh,
-                                        elem,
-                                        mapping,
-                                        intorder,
-                                        quadrature,
-                                        mesh.refdom,
-                                        dofs)
-
-        self.basis = [self.elem.gbasis(self.mapping, self.X, j, tind=elements)
-                      for j in range(self.Nbfun)]
+        super(CellBasis, self).__init__(
+            mesh,
+            elem,
+            mapping,
+            intorder,
+            quadrature,
+            mesh.refdom,
+            dofs,
+        )
 
         if elements is None:
-            self.nelems = mesh.nelements
             self.tind = None
+            self.nelems = mesh.nelements
         else:
-            self.nelems = len(elements)
-            self.tind = self._normalize_elements(elements)
+            self.tind = mesh.normalize_elements(elements)
+            self.nelems = len(self.tind)
 
-        self.dx = (np.abs(self.mapping.detDF(self.X, tind=elements))
+        self.basis = [self.elem.gbasis(self.mapping, self.X, j, tind=self.tind)
+                      for j in range(self.Nbfun)]
+
+        self.dx = (np.abs(self.mapping.detDF(self.X, tind=self.tind))
                    * np.tile(self.W, (self.nelems, 1)))
         logger.info("Initializing finished.")
 
@@ -120,7 +128,13 @@ class CellBasis(AbstractBasis):
 
         # interpolate some previous discrete function at the vertices
         # of the refined mesh
-        w = 0. * x[0]
+        test = self.elem.gbasis(self.mapping, X, 0)[0]
+        if len(test.shape) == 3:
+            w = 0. * x
+        elif len(test.shape) == 2:
+            w = 0. * x[0]
+        else:
+            raise NotImplementedError
         for j in range(self.Nbfun):
             basis = self.elem.gbasis(self.mapping, X, j)
             w += y[self.element_dofs[j]][:, None] * basis[0]
@@ -145,7 +159,7 @@ class CellBasis(AbstractBasis):
 
         return M, w.flatten()
 
-    def probes(self, x: ndarray) -> coo_matrix:
+    def probes(self, x: ndarray):
         """Return matrix which acts on a solution vector to find its values
         on points `x`.
 
@@ -156,12 +170,11 @@ class CellBasis(AbstractBasis):
         not assembled with the usual quadratures.
 
         """
-
         cells = self.mesh.element_finder(mapping=self.mapping)(*x)
         pts = self.mapping.invF(x[:, :, np.newaxis], tind=cells)
         phis = np.array(
             [
-                self.elem.gbasis(self.mapping, pts, k, tind=cells)[0][0]
+                self.elem.gbasis(self.mapping, pts, k, tind=cells)[0]
                 for k in range(self.Nbfun)
             ]
         ).flatten()
@@ -194,7 +207,17 @@ class CellBasis(AbstractBasis):
         values of the given solution vector `y` on given points."""
 
         def interpfun(x: ndarray) -> ndarray:
-            return self.probes(x) @ y
+            # reshape to 2-array to support trailing axes
+            # this is useful, e.g., to pass interpfun to Basis.project
+            shape = None
+            if len(x.shape) > 2:
+                shape = x.shape
+                x = x.reshape(shape[0], -1)
+            out = self.probes(x) @ y
+            # reshape output back to original shape
+            if shape is not None:
+                return out.reshape(*shape[1:])
+            return out
 
         return interpfun
 
@@ -208,6 +231,25 @@ class CellBasis(AbstractBasis):
             elements=self.tind,
         )
 
+    def boundary(self, facets: Optional[Any] = None):
+        """Return corresponding :class:`~skfem.assembly.basis.FacetBasis`.
+
+        Parameters
+        ----------
+        facets
+            Anything that can be passed to ``FacetBasis(..., facets=facets)``.
+
+        """
+        from skfem.assembly.basis.facet_basis import FacetBasis
+        if self.tind is not None:
+            raise NotImplementedError("Boundary of subdomain not supported.")
+        return FacetBasis(
+            self.mesh,
+            self.elem,
+            mapping=self.mapping,
+            facets=facets,
+        )
+
     def project(self, interp, elements=None):
         """Perform :math:`L^2` projection onto the basis.
 
