@@ -27,7 +27,7 @@ class TrilinearForm(Form):
         dx = ubasis.dx
         wdict = FormExtraParams({
             **ubasis.default_parameters(),
-            **self.dictify(kwargs, ubasis),
+            **self._normalize_asm_kwargs(kwargs, ubasis),
         })
 
         # initialize COO data structures
