@@ -3,7 +3,7 @@ sense.
 
 In order to use an element, you simply initialize the respective object and
 pass it to the constructor of :class:`~skfem.assembly.CellBasis` or
-:class:`~skfem.assembly.BoundaryFacetBasis`.  See below for a list of supported
+:class:`~skfem.assembly.FacetBasis`.  See below for a list of supported
 elements.
 
 """
@@ -15,22 +15,22 @@ from .element_hdiv import ElementHdiv
 from .element_hcurl import ElementHcurl
 from .element_global import ElementGlobal
 from .element_vector import ElementVector
-from .element_tri import (ElementTriP1, ElementTriP2,  # noqa
-                          ElementTriP0, ElementTriRT0, ElementTriMorley,
-                          ElementTriArgyris, ElementTriMini, ElementTriCR,
-                          ElementTriHermite, ElementTriCCR,
+from .element_tri import (ElementTriP1, ElementTriP2, ElementTriP0,
+                          ElementTriRT1, ElementTriMorley, ElementTriArgyris,
+                          ElementTriP1B, ElementTriCR, ElementTriHermite,
                           ElementTriP1DG, ElementTriSkeletonP0,
-                          ElementTriSkeletonP1, ElementTriP3,
-                          ElementTriP4, ElementTri15ParamPlate,
-                          ElementTriBDM1)
+                          ElementTriSkeletonP1, ElementTriP3, ElementTriP4,
+                          ElementTri15ParamPlate, ElementTriBDM1,
+                          ElementTriP1G, ElementTriP2G, ElementTriP2B,
+                          ElementTriRT2)  # noqa
 from .element_quad import (ElementQuad0, ElementQuad1, ElementQuad2,  # noqa
                            ElementQuadS2, ElementQuadP,
-                           ElementQuadBFS, ElementQuadRT0, ElementQuad1DG)
+                           ElementQuadBFS, ElementQuadRT1, ElementQuad1DG)
 from .element_tet import (ElementTetP0, ElementTetP1, ElementTetP2,  # noqa
-                          ElementTetRT0, ElementTetN0, ElementTetMini,
+                          ElementTetRT1, ElementTetN0, ElementTetMini,
                           ElementTetCR, ElementTetCCR)
 from .element_hex import (ElementHex0, ElementHex1, ElementHex2,  # noqa
-                          ElementHexS2, ElementHex1DG)
+                          ElementHexS2, ElementHex1DG, ElementHexRT1)
 from .element_line import (ElementLineP0, ElementLineP1, ElementLineP2,  # noqa
                            ElementLinePp, ElementLineHermite,
                            ElementLineMini, ElementLineP1DG)
@@ -60,6 +60,11 @@ ElementTriDG = ElementDG
 ElementQuadDG = ElementDG
 ElementHexDG = ElementDG
 ElementTetDG = ElementDG
+ElementTriMini = ElementTriP1B
+ElementTriCCR = ElementTriP2B
+ElementTriRT0 = ElementTriRT1
+ElementTetRT0 = ElementTetRT1
+ElementQuadRT0 = ElementQuadRT1
 
 
 __all__ = [
@@ -82,6 +87,8 @@ __all__ = [
     "ElementTriCR",
     "ElementTriCCR",
     "ElementTriRT0",
+    "ElementTriRT1",
+    "ElementTriRT2",
     "ElementTriMorley",
     "ElementTri15ParamPlate",
     "ElementTriArgyris",
@@ -91,6 +98,10 @@ __all__ = [
     "ElementTriSkeletonP0",
     "ElementTriSkeletonP1",
     "ElementTriBDM1",
+    "ElementTriP1G",
+    "ElementTriP2G",
+    "ElementTriP1B",
+    "ElementTriP2B",
     "ElementQuad0",
     "ElementQuad1",
     "ElementQuad2",
@@ -99,11 +110,13 @@ __all__ = [
     "ElementQuadP",
     "ElementQuadBFS",
     "ElementQuadRT0",
+    "ElementQuadRT1",
     "ElementQuad1DG",
     "ElementTetP0",
     "ElementTetP1",
     "ElementTetP2",
     "ElementTetRT0",
+    "ElementTetRT1",
     "ElementTetN0",
     "ElementTetMini",
     "ElementTetCR",
@@ -115,6 +128,7 @@ __all__ = [
     "ElementHexS2",
     "ElementHex1DG",
     "ElementHexDG",
+    "ElementHexRT1",
     "ElementLineP0",
     "ElementLineP1",
     "ElementLineP1DG",
