@@ -2,12 +2,14 @@ import logging
 import importlib
 
 from dataclasses import dataclass, replace
+from os import PathLike
 from typing import Callable, Dict, List, Optional, Tuple, Type, Union
 
 import numpy as np
 from numpy import ndarray
 
 from ..element import BOUNDARY_ELEMENT_MAP, Element
+from ..generic_utils import OrientedBoundary
 
 
 logger = logging.getLogger(__name__)
@@ -76,7 +78,9 @@ class Mesh:
 
     @property
     def nedges(self):
-        return self.edges.shape[1]
+        if self.edges is not None:
+            return self.edges.shape[1]
+        raise NotImplementedError
 
     @property
     def nnodes(self):
@@ -146,16 +150,19 @@ class Mesh:
         return edge_candidates[ix]
 
     def with_boundaries(self,
-                        boundaries: Dict[str, Callable[[ndarray], ndarray]],
+                        boundaries: Dict[str, Union[Callable[[ndarray],
+                                                             ndarray],
+                                                    ndarray]],
                         boundaries_only: bool = True):
         """Return a copy of the mesh with named boundaries.
 
         Parameters
         ----------
         boundaries
-            A dictionary of lambda functions with the names of the boundaries
-            as keys.  The midpoint of the facet should return ``True`` for the
-            corresponding lambda function if the facet belongs to the boundary.
+            A dictionary of index arrays or lambda functions with the names of
+            the boundaries as keys.  The midpoint of the facet should return
+            ``True`` for the corresponding lambda function if the facet belongs
+            to the boundary.
         boundaries_only
             If ``True``, consider only facets on the boundary of the domain.
 
@@ -164,8 +171,9 @@ class Mesh:
             self,
             _boundaries={
                 **({} if self._boundaries is None else self._boundaries),
-                **{name: self.facets_satisfying(test, boundaries_only)
-                   for name, test in boundaries.items()}
+                **{name: self.facets_satisfying(test_or_set, boundaries_only)
+                   if callable(test_or_set) else test_or_set
+                   for name, test_or_set in boundaries.items()}
             },
         )
 
@@ -219,6 +227,37 @@ class Mesh:
         subdomains = {} if self._subdomains is None else self._subdomains
         boundaries = {} if self._boundaries is None else self._boundaries
 
+        def encode_boundary(boundary: Union[ndarray, OrientedBoundary]
+                            ) -> ndarray:
+            """return an array with an int per cell encoding a 'boundary'
+
+            i.e. a subset of (optionally oriented) facets.
+
+            Although these are stored internally as arrays of indices of
+            facets, with an optional array of 0/1 orientations, an alternative
+            representation is as a boolean array of shape (self.nelements,
+            self.refdom.nfacets) with entries `True` if that facet belongs to
+            the boundary.  This array can be bit-packed to give a one-
+            dimensional array of ints, one per cell.  This is convenient as it
+            can be treated as `cell_data` by `skfem.io.meshio` and stored in
+            many external formats (VTK, XDMF, ...).
+
+            That is, the binary expansion of the int for a cell gives the bit-
+            flags for whether the corresponding facets are in `boundary`.  This
+            naturally implies the orientation, as the cell is on the inside of
+            the facet.
+            """
+            b = (
+                boundary
+                if isinstance(boundary, OrientedBoundary)
+                else OrientedBoundary(boundary, np.zeros_like(boundary))
+            )
+            t2f_mask = np.zeros_like(self.t2f)
+            columns = self.f2t[(b.ori, b)]
+            r, c = np.nonzero(self.t2f[:, columns] == b)
+            t2f_mask[(r, columns[c])] = 1
+            return (1 << np.arange(self.refdom.nfacets)) @ t2f_mask
+
         return {
             **{
                 f"skfem:s:{name}": [
@@ -227,10 +266,7 @@ class Mesh:
                 for name, subdomain in subdomains.items()
             },
             **{
-                f"skfem:b:{name}": [
-                    ((1 << np.arange(self.t2f.shape[0]))
-                     @ np.isin(self.t2f, boundary)).astype(int)
-                ]
+                f"skfem:b:{name}": [encode_boundary(boundary)]
                 for name, boundary in boundaries.items()
             },
         }
@@ -247,10 +283,16 @@ class Mesh:
             if subnames[1] == "s":
                 subdomains[subnames[2]] = np.nonzero(data[0])[0]
             elif subnames[1] == "b":
-                boundaries[subnames[2]] = np.sort(self.t2f[
-                    (1 << np.arange(self.t2f.shape[0]))[:, None]
-                    & data[0].astype(np.int64) > 0
-                ])
+                mask = (
+                    (1 << np.arange(self.refdom.nfacets))[:, None]
+                    & data[0].astype(int)
+                ).astype(bool)
+                facets = np.sort(self.t2f[mask])
+                cells = mask.nonzero()[1]
+                ori = np.arange(2) @ (self.f2t[:, facets] == cells)
+                boundaries[subnames[2]] = (
+                    OrientedBoundary(facets, ori) if ori.any() else facets
+                )
 
         return boundaries, subdomains
 
@@ -284,7 +326,8 @@ class Mesh:
 
     def facets_satisfying(self,
                           test: Callable[[ndarray], ndarray],
-                          boundaries_only: bool = False) -> ndarray:
+                          boundaries_only: bool = False,
+                          normal: Optional[ndarray] = None) -> ndarray:
         """Return facets whose midpoints satisfy some condition.
 
         Parameters
@@ -294,15 +337,46 @@ class Mesh:
             to be included in the return set.
         boundaries_only
             If ``True``, include only boundary facets.
+        normal
+            If given, used to orient the set of facets.
 
         """
-        midp = [np.sum(self.p[itr, self.facets], axis=0) / self.facets.shape[0]
-                for itr in range(self.dim())]
-        facets = np.nonzero(test(np.array(midp)))[0]
+        midp = self.p[:, self.facets].mean(axis=1)
+        facets = np.nonzero(test(midp))[0]
         if boundaries_only:
             facets = np.intersect1d(facets, self.boundary_facets())
+        if normal is not None:
+            tind = self.f2t[0, facets]
+            mapping = self._mapping()
+            normals = mapping.normals(np.zeros((self.dim(), 1)),
+                                      tind,
+                                      facets,
+                                      self.t2f).T[0].T
+            ori = 1 * (np.dot(normal, normals) < 0)
+            return OrientedBoundary(facets, ori)
         return facets
 
+    def facets_around(self, elements, flip=False) -> OrientedBoundary:
+        """Return the oriented set of facets around a set of elements.
+
+        Parameters
+        ----------
+        elements
+            An array of element indices or, alternatively, something that can
+            be cast into one via ``Mesh.normalize_elements``.
+        flip
+            If ``True``, use traces outside the subdomain and inward normals.
+
+        """
+        elements = self.normalize_elements(elements)
+        facets, counts = np.unique(self.t2f[:, elements], return_counts=True)
+        facets = facets[counts == 1]
+        if flip:
+            ori = np.nonzero(~np.isin(self.f2t[:, facets], elements).T)[1]
+        else:
+            ori = np.nonzero(np.isin(self.f2t[:, facets], elements).T)[1]
+        return OrientedBoundary(facets, ori)
+
     def elements_satisfying(self,
                             test: Callable[[ndarray], ndarray]) -> ndarray:
         """Return elements whose midpoints satisfy some condition.
@@ -314,9 +388,8 @@ class Mesh:
             are to be included in the return set.
 
         """
-        midp = [np.sum(self.p[itr, self.t], axis=0) / self.t.shape[0]
-                for itr in range(self.dim())]
-        return np.nonzero(test(np.array(midp)))[0]
+        midp = self.p[:, self.t].mean(axis=1)
+        return np.nonzero(test(midp))[0]
 
     def _expand_facets(self, ix: ndarray) -> Tuple[ndarray, ndarray]:
         """Return vertices and edges corresponding to given facet indices.
@@ -557,7 +630,7 @@ class Mesh:
         return self.__repr__()
 
     def save(self,
-             filename: str,
+             filename: Union[str, PathLike],
              point_data: Optional[Dict[str, ndarray]] = None,
              cell_data: Optional[Dict[str, ndarray]] = None,
              **kwargs) -> None:
@@ -914,12 +987,95 @@ class Mesh:
         """
         raise NotImplementedError
 
-    def draw(self, *args, **kwargs):
-        """Convenience wrapper for vedo."""
-        if 'visuals' in kwargs:
-            visuals = kwargs['visuals']
-            del kwargs['visuals']
-        else:
-            visuals = 'vedo'
+    def draw(self, visuals='matplotlib', **kwargs):
+        """Convenience wrapper for skfem.visuals."""
+        if not isinstance(visuals, str):
+            logger.warning("First argument, 'visuals', must be a string.")
         mod = importlib.import_module('skfem.visuals.{}'.format(visuals))
-        return mod.draw(self, *args, **kwargs)
+        return mod.draw(self, **kwargs)
+
+    def plot(self, x, visuals='matplotlib', **kwargs):
+        """Convenience wrapper for skfem.visuals."""
+        if not isinstance(visuals, str):
+            logger.warning("Second argument, 'visuals', must be a string.")
+        mod = importlib.import_module('skfem.visuals.{}'.format(visuals))
+        return mod.plot(self, x, **kwargs)
+
+    def normalize_facets(self, facets) -> ndarray:
+        """Generate an array of facet indices.
+
+        Parameters
+        ----------
+        elements
+            Criteria of which facets to include.  Function has different
+            behavior based on the type of this parameter.
+
+        """
+        if isinstance(facets, int):
+            # Make  normalize_facets([1,2,3]) have the same behavior as
+            # normalize_facets(np.array([1,2,3]))
+            return np.array([facets])
+        if isinstance(facets, ndarray):
+            # Assume the facets have already been normalized
+            return facets
+        if facets is None:
+            # Default behavior.
+            return self.boundary_facets()
+        elif isinstance(facets, (tuple, list, set)):
+            # Recurse over the list, building an array of all matching facets
+            return np.unique(
+                np.concatenate(
+                    [self.normalize_facets(f) for f in facets]
+                )
+            )
+        elif callable(facets):
+            # The callable should accept an array of facet centers and return
+            # an boolean array with True for facets that should be included.
+            return self.facets_satisfying(facets)
+        elif isinstance(facets, str):
+            # Assume string is the label of a boundary in the mesh.
+            if ((self.boundaries is not None
+                 and facets in self.boundaries)):
+                return self.boundaries[facets]
+            else:
+                raise ValueError("Boundary '{}' not found.".format(facets))
+        raise NotImplementedError
+
+    def normalize_elements(self, elements) -> ndarray:
+        """Generate an array of element indices.
+
+        Parameters
+        ----------
+        elements
+            Criteria of which elements to include.  Function has different
+            behavior based on the type of this parameter.
+
+        """
+        if isinstance(elements, bool) and elements:
+            return np.arange(self.nelements, dtype=np.int64)
+        if isinstance(elements, int):
+            # Make  normalize_elements([1,2,3]) have the same behavior as
+            # normalize_elements(np.array([1,2,3]))
+            return np.array([elements])
+        if isinstance(elements, ndarray):
+            # Assume the elements have already been normalized
+            return elements
+        if callable(elements):
+            # The callable should accept an array of element centers and return
+            # an boolean array with True for elements that should be included.
+            return self.elements_satisfying(elements)
+        elif isinstance(elements, (tuple, list, set)):
+            # Recurse over the list, building an array of all matching elements
+            return np.unique(
+                np.concatenate(
+                    [self.normalize_elements(e) for e in elements]
+                )
+            )
+        elif isinstance(elements, str):
+            # Assume string is the label of a subdomain in the mesh.
+            if ((self.subdomains is not None
+                 and elements in self.subdomains)):
+                return self.subdomains[elements]
+            else:
+                raise ValueError("Subdomain '{}' not found.".format(elements))
+        raise NotImplementedError
